#include "database.h"

#define maxRec 1000

Dbrec db[maxRec];                 /* the array of records */
int numRecs=0;                    /* the current number of records */

int findRec(dbkeytype key, dbdatatype *data, indextype *idx)
/* Preconditions: The keys of the records are in order. numRecs >= 0. 
   Action: Find key in the list of records. Returns whether or not 
           it was found, the data if found, and a pointer to the
           corresponding record if found, or to the record whose key
           immediately comes after key if not found. */
{
  unsigned int low=0,
               high=numRecs,
	       found=0,
	       mid;
  int cmp;

  while(!found && low<high) {
    /* LI: (1) All keys from db[0].key to db[low-1].key are less 
               than key.
           (2) All keys from db[high].key to db[numRecs-1].key are 
               greater than key.
           (3) If found, then db[mid].key = key.
           (4) low < high. 
	   (5) low <= mid < high.
	   */
    mid=low + (high-low)/2;
    cmp=comparedbkey(key,db[mid].key);
    if(cmp<0)
      high=mid;
    else if(cmp>0)
      low=mid+1;
    else /* cmp==0 */
      found=1;
  }
  if(found) {
    /* By LI(3), mid is the record with key key. */
    *idx = db+mid;
    copydbdata((*idx)->data,*data);
  }
  else
    /* If !found, then low>=high, so low==high by LI(4) and LI(5). By LI(1) and LI(2),
       low is the record whose key comes immediately after key. */
    *idx = db+low;
  return found;
}

int createRec(dbkeytype key, dbdatatype data, indextype idx)
/* Precondition: idx is where the new record should go.
   Action: If the database is not full, adds the record with key key and
           data data, and return 1. Otherwise, return 0. */
{
  Dbrec *rec;

  if (numRecs==maxRec)
    return 0;
  for(rec=db+numRecs;rec>idx;rec--)
    *rec = *(rec-1);
  copydbkey(key,idx->key);
  copydbdata(data,idx->data);
  numRecs++;
  return 1;
}

void setRec(indextype idx, dbdatatype data)
/* Precondition: idx is a pointer to a database record.
   Action: Set the data of the record pointed to be idx to be data. */
{
  copydbdata(data,idx->data);
}

void eachRec(void (*fn)(dbkeytype,dbdatatype))
/* Action: Call the function on each key and data stored in the
   database. */
{
  Dbrec *rec;

  for(rec=db;rec<db+numRecs;rec++)
    (*fn)(rec->key,rec->data);
}
