#include <stdio.h>
#include <stdlib.h>

#define MIS   1
#define CAN   4
#define BOAT  16
#define ALL   BOAT+2*CAN+3*MIS
#define NONE  0
#define SUBSET(x,y) ((x)%4<=(y)%4 && (x)/4%4<=(y)/4%4 && (x)/16<=(y)/16)

int n,position[ALL+2],visited[ALL+1];

void printSide(int p)
{
  printf("%*dM %dC %dB\n",n+1,p%4,(p/4)%4,p/16);
}

int validMove(int pos1, int pos2)
/* Returns 1 if move is valid from pos1 to pos2, pos2 is legal, and
   has not already been seen. Otherwise returns 0. */
{
  int diff = (SUBSET(pos2,pos1)?pos1-pos2:       /* change in position */
              (SUBSET(pos1,pos2)?pos2-pos1:0));
  int mis2 = pos2%4;                             /* missionaries in pos2 */
  int can2 = pos2/4%4;                           /* cannibals in pos2 */

  return
    (!visited[pos2]                /* have not been there already */
     && (diff==BOAT+MIS || diff==BOAT+MIS+CAN || diff==BOAT+2*MIS)
                                   /* valid move */
     && can2 <= 2                  /* pos2 has at most two cannibals */
     && (mis2 == 0 || can2 <= mis2)/* left bank missionaries uneaten */
     && (3-mis2 == 0 || 2-can2 <= 3-mis2)); /* right bank mis. uneaten */
}

int main()
{
  position[0] = ALL;               /* initial position is under consideration */
  visited[ALL] = 1;                
  printSide(position[0]);
  position[1] = NONE;              /* start with the first possible next move */
  n = 1;
  while(position[n-1]!=NONE) {
    while(position[n]<=ALL && !validMove(position[n-1],position[n]))
      position[n]++;               /* loop until a valid next move, or no move */
    if(position[n]<=ALL) {         /* there is a valid next move */
      visited[position[n]] = 1;    /* mark that it is being considered */
      printSide(position[n]);
      position[++n] = NONE;        /* start its next moves at first possible one */
    }
    else {                         /* no valid next move */
      visited[position[--n]] = 0;  /* no longer considered prev state, backtrack */
      if(n==0) return 0;
      position[n]++;               /* advance previous state's next move */
    }
  }
  return 1;
}
